#!/usr/bin/python
import sys
import errno

from bzrlib import plugin
from bzrlib.branch import Branch
from jenkinsapi.jenkins import Jenkins

# Load plugins so we can use lp: URLs
plugin.load_plugins()

JENKINS_URL = 'http://d-jenkins.ubuntu-ci:8080'


def branch_url_to_filename(branch_url):
    return branch_url.replace('/', '-')


def get_current_revno(branch_url):
    branch = Branch.open(branch_url)
    return branch.revno()


def get_last_revno(branch_url):
    try:
        last_revno = open(branch_url_to_filename(branch_url), "rb").read()
    except IOError as e:
        if e.errno == errno.ENOENT:
            # Can't find the file, assume it's the first run.
            last_revno = 0
        else:
            raise
    return int(last_revno)


def save_revno(branch_url, revno):
    with open(branch_url_to_filename(branch_url), 'wb') as f:
        f.write(str(revno))


def main(argv):
    # Usage:
    # jenkinstrigger.py <job-name> <job-token>
    job_name = argv[1]
    job_token = argv[2]
    jenkins = Jenkins(JENKINS_URL)
    job = jenkins.get_job(job_name)
    for param in job.get_params():
        if param.get('name') == 'MAAS_BRANCH':
            maas_branch_url = param.get('defaultParameterValue')['value']
        elif param.get('name') == 'PACKAGING_BRANCH':
            packaging_url = param.get('defaultParameterValue')['value']

    current_maas_revno = get_current_revno(maas_branch_url)
    current_packaging_revno = get_current_revno(packaging_url)
    last_maas_revno = get_last_revno(maas_branch_url)
    last_packaging_revno = get_last_revno(packaging_url)

    if (current_maas_revno > last_maas_revno or
        current_packaging_revno > last_packaging_revno):

        params = {
            'MAAS_BRANCH_REVNO': current_maas_revno,
            'PACKAGING_REVNO': current_packaging_revno,
            'MAAS_BRANCH': maas_branch_url,
            'PACKAGING_BRANCH': packaging_url
        }
        print "Building %s with %s" % (job_name, params)
        job.invoke(securitytoken=job_token, build_params=params)
        save_revno(maas_branch_url, current_maas_revno)
        save_revno(packaging_url, current_packaging_revno)
    else:
        print ("Not running tests because:\n"
            "%s (revno: %s) <= %s\n"
            "%s (revno: %s) <= %s\n" % (
            maas_branch_url, current_maas_revno, last_maas_revno,
            packaging_url, current_packaging_revno, last_packaging_revno))


if __name__ == '__main__':
    main(sys.argv)
